@@ -559,35 +559,34 @@ static ret_t
 send_post (cherokee_handler_proxy_t *hdl)
 {
 	ret_t                  ret;
-	int                    eagain_fd = -1;
-	int                    mode      =  0;
-	cherokee_connection_t *conn      = HANDLER_CONN(hdl);
-		
-	ret = cherokee_post_walk_to_fd (&conn->post,
-					hdl->pconn->socket.socket,
-					&eagain_fd, &mode);
+	cherokee_connection_t *conn = HANDLER_CONN(hdl);
+
+	ret = cherokee_post_walk_to_socket (&conn->post, &hdl->pconn->socket);
 
 	TRACE (ENTRIES",post", "Sending POST fd=%d, ret=%d\n", 
 	       hdl->pconn->socket.socket, ret);
-	
+
 	switch (ret) {
 	case ret_ok:
 		TRACE (ENTRIES",post", "%s\n", "finished");
 		return ret_ok;
-		
+
+	case ret_eof:
+	case ret_error:
+		return ret;
+
 	case ret_eagain:
-		if (eagain_fd != -1) {
-			ret = cherokee_thread_deactive_to_polling (HANDLER_THREAD(hdl),
-								   conn, eagain_fd,
-								   mode, false);
-			if (ret != ret_ok) {
-				return ret_eof;
-			}
+		ret = cherokee_thread_deactive_to_polling (HANDLER_THREAD(hdl),
+							   conn, hdl->pconn->socket.socket,
+							   FDPOLL_MODE_WRITE, false);
+		if (ret != ret_ok) {
+			return ret_eof;
 		}
 		return ret_eagain;
 		
 	default:
-		return ret;
+		RET_UNKNOWN(ret);
+		return ret_error;
 	}	
 }
 
@@ -761,7 +760,13 @@ cherokee_handler_proxy_init (cherokee_handler_proxy_t *hdl)
 		ret = cherokee_handler_proxy_conn_recv_headers (hdl->pconn, &hdl->tmp);
 		switch (ret) {
 		case ret_ok:
-			/* Got the header */
+			/* Turn Error Handling on. From this point the
+			 * handler object supports error handling.
+			 */
+			HANDLER(hdl)->support |= hsupport_error;
+			
+			/* Got the header.
+			 */
 			break;
 		case ret_eagain:
 			ret = cherokee_thread_deactive_to_polling (HANDLER_THREAD(hdl),
@@ -814,6 +819,7 @@ parse_server_header (cherokee_handler_proxy_t *hdl,
 	char                           *header_end;
 	cherokee_list_t                *i;
 	cherokee_http_version_t         version;
+	cherokee_boolean_t              encoded      = false;
 	cherokee_connection_t          *conn         = HANDLER_CONN(hdl);
 	cherokee_handler_proxy_props_t *props        = HDL_PROXY_PROPS(hdl);
 
@@ -925,6 +931,15 @@ parse_server_header (cherokee_handler_proxy_t *hdl,
 				cherokee_buffer_add_str    (buf_out, CRLF);
 				goto next;
 			}
+
+		} else if (strncmp (begin, "Content-Encoding:", 17) == 0) {
+			encoded = true;
+
+			if (conn->encoder != NULL) {
+				cherokee_encoder_free (conn->encoder);
+				conn->encoder = NULL;
+			}
+
 		} else {
 			colon = strchr (begin, ':');
 			if (unlikely (colon == NULL)) {
@@ -955,6 +970,14 @@ parse_server_header (cherokee_handler_proxy_t *hdl,
 		add_header (buf_out, &HEADER_ADD(i)->key, &HEADER_ADD(i)->val);
 	}
 
+	/* Deal with Content-Encoding: If the response is no encoded,
+	 * and the handler is configured to encode, it has to add the
+	 * encoder headers at this point.
+	 */
+	if ((! encoded) && (conn->encoder)) {
+		cherokee_encoder_add_headers (conn->encoder, buf_out);		
+	}
+
 	/* Special case: Client uses Keepalive, the back-end sent a
 	 * no-body reply with no Content-Length.
 	 */
@@ -1245,7 +1268,7 @@ cherokee_handler_proxy_new (cherokee_handler_t     **hdl,
 	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_proxy_step;
 	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_proxy_add_headers;
 
-	HANDLER(n)->support = hsupport_full_headers | hsupport_error;
+	HANDLER(n)->support = hsupport_full_headers;
 
 	/* Init
 	 */
